package com.objectriver.runtime.webrpc.xdr.marshals;

import com.objectriver.runtime.webrpc.exception.WebRpcMarshallingException;
import com.objectriver.runtime.webrpc.xdr.XDR;
import com.objectriver.runtime.webrpc.xdr.XdrProcT;
import com.objectriver.runtime.webrpc.xdr.XdrThing;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.BufferOverflowException;
import java.nio.BufferUnderflowException;

/**
 * ObjectRiver Inc, http://www.objectriver.net/
 * Copyright (c) 2002-2013, ObjectRiver(tm). All rights reserved.
 */
public class xdr_string extends XdrMarshal {
    public static xdr_string instance;
    static {
        instance = new xdr_string();
    }
    private xdr_string() {}
    public boolean execute(XDR xdrs, XdrThing thing, boolean notnull) throws WebRpcMarshallingException {
        if (notnull && xdrs.x_op == XDR.op.XDR_ENCODE) {
            if(thing.isNull()) throw new WebRpcMarshallingException("xdr_string() string type can not be null! try defining as String.");
        }
        return execute(xdrs,thing);
    }
    public boolean execute(XDR xdrs, XdrThing thing) throws WebRpcMarshallingException {
        Integer size=0;
        String string=null;
        /**
         * First deal with length of string.
         */
        try {
            if (xdrs.x_op == XDR.op.XDR_ENCODE) {
                string = (String)thing.getThing();
                size = ( string==null )? 0 : string.length();
            }
            // ENCODE DECODE
            XdrThing thing2 = new XdrThing(size);
            if(!xdr_int.instance.execute(xdrs,thing2)) {
                return false;
            }
            size = (Integer)thing2.getThing();
//            if(size>thing.getMaxsize()) {
//                return false;
//            }
            /**
             * Now deal with marshalling string.
             */
            switch(xdrs.x_op) {
            case XDR_DECODE:
                if(size==0) {
                    thing.setThing("");
                    return true;
                }
                byte[] bytes = new byte[size];
                if(xdr_opaque(xdrs, bytes ,size )) {
                    //String decoded = String.valueOf(bytes);
                    String decoded = new String(bytes,"UTF-8");
                    thing.setThing(decoded);
                    return true;
                }
                return false;
            case XDR_ENCODE: {
                if(size==0) {
                    return true;
                }
                return xdr_opaque(xdrs, string.getBytes() ,size );
            }
            }
        }
        catch (UnsupportedEncodingException ex) { // new String(bytes,"UTF-8");
            throw new WebRpcMarshallingException("xdr_string", ex);
        }
        catch (BufferUnderflowException ex) {
            throw new WebRpcMarshallingException("xdr_string", ex);
        }
        catch (BufferOverflowException ex) {
            throw new WebRpcMarshallingException("xdr_string", ex);
        }

        return false;
    }
    public Class getClazz() {
        return String.class;
    }
}
